from ._yref import YrefValidator
from ._yanchor import YanchorValidator
from ._y import YValidator
from ._xref import XrefValidator
from ._xanchor import XanchorValidator
from ._x import XValidator
from ._visible import VisibleValidator
from ._templateitemname import TemplateitemnameValidator
from ._source import SourceValidator
from ._sizing import SizingValidator
from ._sizey import SizeyValidator
from ._sizex import SizexValidator
from ._opacity import OpacityValidator
from ._name import NameValidator
from ._layer import LayerValidator
